/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
package org.springframework.osgi.config;

import java.util.StringTokenizer;
import java.util.List;
import java.util.HashSet;
import java.util.Iterator;

import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.config.RuntimeBeanReference;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.ManagedList;
import org.springframework.beans.factory.xml.AbstractBeanDefinitionParser;
import org.springframework.beans.factory.xml.ParserContext;
import org.springframework.beans.MutablePropertyValues;
import org.springframework.core.Conventions;
import org.springframework.osgi.service.OsgiServiceProxyFactoryBean;
import org.springframework.osgi.bundle.support.PackageSpecification;
import org.springframework.util.StringUtils;
import org.springframework.util.xml.DomUtils;
import org.w3c.dom.Attr;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;

/**
 * @author Andy Piper
 * @since 2.1
 */
class ReferenceBeanDefinitionParser extends AbstractBeanDefinitionParser
{
	public final static String LISTENER_ID = "listeners";
	public final static String DEPENDS_ON = "depends-on";
	public static final String LAZY_INIT = "lazy-init";
	public static final String PROPERTIES = "properties";

	protected BeanDefinition parseInternal(Element element, ParserContext parserContext) {
		BeanDefinitionBuilder builder = BeanDefinitionBuilder.rootBeanDefinition(OsgiServiceProxyFactoryBean.class);
		NamedNodeMap attributes = element.getAttributes();
		for (int x = 0; x < attributes.getLength(); x++) {
			Attr attribute = (Attr) attributes.item(x);
			String name = attribute.getLocalName();

			if (ID_ATTRIBUTE.equals(name)) {
				continue;
			} else if (DEPENDS_ON.equals(name)) {
				for (StringTokenizer dependents = new StringTokenizer(attribute.getValue(), ", ");
				     dependents.hasMoreElements();) {
					String dep = (String) dependents.nextElement();
					if (StringUtils.hasText(dep)) {
						builder.addDependsOn(dep);
					}
				}
			} else if (LAZY_INIT.equals(name)) {
				builder.setLazyInit(Boolean.getBoolean(attribute.getValue()));
			} else if (LISTENER_ID.equals(name)) {
				ManagedList alist = new ManagedList();
				for (StringTokenizer listeners = new StringTokenizer(attribute.getValue(), ", ");
				     listeners.hasMoreElements();) {
					String l = (String) listeners.nextElement();
					if (StringUtils.hasText(l)) {
						alist.add(new RuntimeBeanReference(l));
					}
				}
				builder.addPropertyValue(LISTENER_ID, alist);
			} else {
				builder.addPropertyValue(Conventions.attributeNameToPropertyName(name), attribute.getValue());
			}
		}

		MutablePropertyValues props = parserContext.getDelegate().parsePropertyElements(element);
		if (props != null && !props.isEmpty()) {
			builder.addPropertyValue(PROPERTIES, props);
		}
		return builder.getBeanDefinition();
	}
}
