/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
package org.springframework.osgi.context;

import java.io.File;
import java.util.ArrayList;
import java.util.Dictionary;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.BundleEvent;
import org.osgi.framework.BundleListener;
import org.osgi.framework.Constants;
import org.osgi.framework.SynchronousBundleListener;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.xml.DefaultNamespaceHandlerResolver;
import org.springframework.beans.factory.xml.NamespaceHandlerResolver;
import org.springframework.beans.factory.xml.PluggableSchemaResolver;
import org.springframework.core.io.ResourceLoader;
import org.springframework.osgi.context.support.AbstractBundleXmlApplicationContext;
import org.springframework.osgi.context.support.DefaultOsgiBundleXmlApplicationContextFactory;
import org.springframework.osgi.context.support.OsgiBundleXmlApplicationContextFactory;
import org.springframework.osgi.context.support.OsgiResourceUtils;
import org.springframework.util.ResourceUtils;
import org.springframework.util.StringUtils;
import org.xml.sax.EntityResolver;

/**
 * @author Bill Gallagher
 * @author Andy Piper
 * @author Hal Hildebrand
 */
public class ContextLoaderListener implements BundleActivator, SynchronousBundleListener
{
	// The standard for META-INF header keys excludes ".", so these constants must use "-"
	private static final String CONTEXT_LOCATION_HEADER = "org-springframework-context";
	//private static final String PARENT_CONTEXT_SERVICE_NAME_HEADER = "org.springframework.parent.context";
	private static final String CONTEXT_LOCATION_DELIMITERS = ", ";
	private static final String DEFAULT_CONTEXT_PREFIX = "/META-INF/";
	private static final String DEFAULT_GLOBAL_CONFIG = "spring.xml";
	private static final String GLOBAL_CONFIG_PROP = "org.springframework.osgi.config";
	private static final String DEFAULT_CONTEXT_POSTFIX = "-context.xml";
	private Log log = LogFactory.getLog(ContextLoaderListener.class);

	private AbstractBundleXmlApplicationContext context;
	private long lastModified;
	private Thread updateThread;
	private File globalConfig;
	private long bundleId;

	private OsgiBundleXmlApplicationContextFactory contextFactory = new DefaultOsgiBundleXmlApplicationContextFactory();

	private HashMap managedBundles = new HashMap();

	// Required by the BundleActivator contract
	public ContextLoaderListener() {
		// FIXME andyp -- config should be a comma-separated list of locations.
		String config = System.getProperty(GLOBAL_CONFIG_PROP);
		if (config == null) config = DEFAULT_GLOBAL_CONFIG;
		globalConfig = new File(config);
	}

	public void start(BundleContext context) throws Exception {
		Dictionary props = new Hashtable();
		bundleId = context.getBundle().getBundleId();
		// Register the default namespace handlers as OSGi services so that the OSGi application context
		// implementation will be able to pick them up.
		context.registerService(EntityResolver.class.getName(),
				new PluggableSchemaResolver(ContextLoaderListener.class.getClassLoader()), props);
		context.registerService(NamespaceHandlerResolver.class.getName(),
				new DefaultNamespaceHandlerResolver(ContextLoaderListener.class.getClassLoader()), props);

		context.addBundleListener(this);

		lastModified = globalConfig.lastModified();
		if (globalConfig.exists()) {
			this.context = contextFactory.createApplicationContextWithBundleContext(context,
					new String[]{globalConfig.toURL().toString()}, false);
		}

		updateThread = new UpdateThread();
		updateThread.setDaemon(true);
		updateThread.start();
	}

	public void stop(BundleContext context) throws Exception {
		this.context.close();
	}

	public void bundleChanged(BundleEvent event) {
		if (event.getBundle().getBundleId() == bundleId) return;

		switch (event.getType()) {
			case BundleEvent.STARTED:
				startBundle(event.getBundle());
				break;
			case BundleEvent.STOPPED:
  			stopBundle(event.getBundle());
				break;
		}
	}

	private void startBundle(Bundle bundle) {
		String[] applicationContextLocations = getApplicationContextLocations(bundle);

		BundleContext bundleContext = OsgiResourceUtils.getBundleContext(bundle);
		if (bundleContext == null) {
			log.error("Could not resolve BundleContext for bundle [" + bundle + "]");
			return;
		}
		// No need to set the CCL. OsgiBundleXmlApplicationContext will do this for us.
		try {
			if (log.isInfoEnabled()) {
				log.info("Starting bundle [" + bundle.getHeaders().get(Constants.BUNDLE_NAME) + "] with configuration ["
						+ (applicationContextLocations == null ? "none" : applicationContextLocations[0]) + "]");
			}
			AbstractBundleXmlApplicationContext ctx =
					contextFactory.createApplicationContextWithBundleContext(bundleContext,
							applicationContextLocations, true);
			managedBundles.put(bundle, ctx);
		} catch (Throwable thr) {
			thr.printStackTrace();
		}
	}

	private void stopBundle(Bundle bundle) {
		AbstractBundleXmlApplicationContext ctx = (AbstractBundleXmlApplicationContext) managedBundles.remove(bundle);
		if (ctx != null) {
			ctx.close();
		}
	}

	/**
	 * Retrieves the org.springframework.context manifest header attribute
	 * and parses it to create a String[] of resource names for creating
	 * the application context.
	 * <p/>
	 * If the org.springframework.context header is not present, the
	 * default <bundle-symbolic-name>-context.xml file will be returned.
	 */
	protected String[] getApplicationContextLocations(Bundle bundle) {
		Dictionary manifestHeaders = bundle.getHeaders();
		String contextLocationsHeader = (String) manifestHeaders.get(CONTEXT_LOCATION_HEADER);
		if (contextLocationsHeader != null) {
			String[] locs =
					StringUtils.tokenizeToStringArray(contextLocationsHeader, CONTEXT_LOCATION_DELIMITERS);
			List ret = new ArrayList();
			for (int i = 0; i < locs.length; i++) {
				if (bundle.getEntry(locs[i]) != null) {
					ret.add(locs[i]);
				}
			}
			if (ret.isEmpty())
				return null;
			else
				return addBundlePrefixTo((String[]) ret.toArray());
		} else {
			String defaultName = DEFAULT_CONTEXT_PREFIX + bundle.getSymbolicName() + DEFAULT_CONTEXT_POSTFIX;
			if (bundle.getEntry(defaultName) != null)
				return addBundlePrefixTo(new String[]{defaultName});
			// Didn't find the symbolic name, try the actual name.
			defaultName = DEFAULT_CONTEXT_PREFIX + bundle.getHeaders().get(Constants.BUNDLE_NAME) + DEFAULT_CONTEXT_POSTFIX;
			if (bundle.getEntry(defaultName) != null)
				return addBundlePrefixTo(new String[]{defaultName});
			else
				return null;
		}
	}

	/**
	 * add the "bundle:" prefix to the resource location paths in the given argument.
	 * This ensures that only this bundle will be searched for matching resources.
	 * <p/>
	 * Modifies the argument in place and returns it.
	 */
	private String[] addBundlePrefixTo(String[] resourcePaths) {
		for (int i = 0; i < resourcePaths.length; i++) {
			if (!resourcePaths[i].startsWith(AbstractBundleXmlApplicationContext.BUNDLE_URL_PREFIX)
					&& !resourcePaths[i].startsWith(ResourceLoader.CLASSPATH_URL_PREFIX)
					&& !resourcePaths[i].startsWith(ResourceUtils.FILE_URL_PREFIX)) {
				resourcePaths[i] = AbstractBundleXmlApplicationContext.BUNDLE_URL_PREFIX + resourcePaths[i];
			}
		}
		return resourcePaths;
	}

	private class UpdateThread extends Thread
	{
		private static final int SLEEP_TIME = 500;

		public void run() {
			while (true) {
				if (globalConfig.exists() && (globalConfig.lastModified() != lastModified)) {
					lastModified = globalConfig.lastModified();
					try {
						context.refresh();
					} catch (BeansException be) {
						log.error("Error updating configuration from spring.xml", be);
					}
				}
				try {
					Thread.sleep(SLEEP_TIME);
				} catch (InterruptedException ignore) {
				}
			}
		}
	}
}
