/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
package org.springframework.osgi.context.support;

import java.util.Dictionary;
import java.util.Properties;

import org.springframework.context.support.AbstractXmlApplicationContext;
import org.springframework.context.ApplicationContext;
import org.springframework.beans.factory.xml.XmlBeanDefinitionReader;
import org.springframework.beans.factory.config.ConfigurableListableBeanFactory;
import org.springframework.beans.BeansException;
import org.springframework.core.io.Resource;
import org.springframework.core.io.ResourceLoader;
import org.springframework.core.io.FileSystemResource;
import org.springframework.util.Assert;
import org.springframework.util.ResourceUtils;
import org.springframework.osgi.context.BundleContextAware;
import org.springframework.osgi.service.OsgiServiceUtils;
import org.osgi.framework.BundleContext;
import org.osgi.framework.Bundle;
import org.osgi.framework.ServiceRegistration;
import org.osgi.framework.Constants;
import org.osgi.framework.ServiceReference;

/**
 * Application context backed by an OSGi bundle. Will use the
 * bundle classpath for resource loading for any unqualified resource string.
 * <p/>
 * Also understands the "bundle:" resource prefix for explicit loading
 * of resources from the bundle. When the bundle prefix is used the
 * target resource must be contained within the bundle (or attached
 * fragments), the classpath is not searched.
 * <p/>
 * TODO: provide means to access OSGi services etc. through this application context?
 * TODO: think about whether restricting config files to bundle: is the right thing to do
 * TODO: listen to parent application context service, automatically rebind and refresh if
 * it goes away and comes back
 *
 * @author Adrian Colyer
 * @author Andy Piper
 * @author Hal Hildebrand
 * @since 2.0
 */
public class AbstractBundleXmlApplicationContext extends AbstractXmlApplicationContext
{
	public static final String BUNDLE_URL_PREFIX = "bundle:";
	public static final String APPLICATION_CONTEXT_SERVICE_NAME_HEADER = "org-springframework-context-service-name";
	public static final String NAMESPACE_HANDLER_RESOLVERS_HEADER = "Spring-Namespaces";
	public static final String PARENT_CONTEXT_SERVICE_NAME_HEADER = "SpringParentContext";

	private static final String APPLICATION_CONTEXT_SERVICE_POSTFIX = "-springApplicationContext";

	protected Bundle osgiBundle;
	protected BundleContext osgiBundleContext;
	protected String[] configLocations;
	private ServiceRegistration serviceRegistration;

	public AbstractBundleXmlApplicationContext(BundleContext context, String[] configLocations,
	                                           ClassLoader classLoader)
	{
		this(getParentApplicationContext(context), context, configLocations, classLoader);
	}

	public AbstractBundleXmlApplicationContext(ApplicationContext parent, BundleContext context, String[] configLocations,
	                                           ClassLoader classLoader)
	{
		super(parent);
    this.osgiBundleContext = context;
		this.osgiBundle = context.getBundle();
    this.configLocations = configLocations;
	  this.setClassLoader(classLoader);
		this.setDisplayName("AbstractBundleXmlApplicationContext(bundle="
			+ getBundleName() + ", config=" + getConfigName() + ")");

	}

	protected String[] getConfigLocations() {
	  return this.configLocations;
	}

	private String getConfigName() {
		return configLocations == null ? "<no config>" : configLocations[0];
	}

	protected String getBundleName() {
		return (String)(osgiBundle.getSymbolicName() == null ? osgiBundle.getHeaders().get(Constants.BUNDLE_NAME) :
				osgiBundle.getSymbolicName());
	}

	/**
	 * Get the OSGi BundleContext for this application context
	 */
	public BundleContext getBundleContext() {
	  return this.osgiBundleContext;
	}

	/**
	 * We can't look in META-INF across bundles when using osgi, so we need to
	 * change the default namespace handler (spring.handlers) location with a
	 * custom resolver.  Each Spring OSGi bundle which provides a namespace handler plugin
	 * publishes a NamespaceHandlerResolver service as well as an EntityResolver service
	 * which is used to plug in the namespaces supported by the bundle.
	 */
	protected void initBeanDefinitionReader(XmlBeanDefinitionReader beanDefinitionReader) {
	  beanDefinitionReader.setEntityResolver(new ServiceSchemaResolver(this.osgiBundleContext));
	  beanDefinitionReader.setNamespaceHandlerResolver(new ServiceNamespaceResolver(this.osgiBundleContext));
	}

	/**
	 * The name we will use when publishing this application context as an OSGi service.
	 * If the APPLICATION_CONTEXT_SERVICE_NAME_HEADER manifest header is present, we
	 * use the user given name, otherwise we derive a name from the bundle symbolic name.
	 */
	protected String getServiceName() {
	  String userSpecifiedName = (String) this.osgiBundle.getHeaders().get(APPLICATION_CONTEXT_SERVICE_NAME_HEADER);
	  if (userSpecifiedName != null) {
	    return userSpecifiedName;
	  } else {
	    return getBundleName() + APPLICATION_CONTEXT_SERVICE_POSTFIX;
	  }
	}

	/**
	 * Implementation of getResource that delegates to the bundle for
	 * any unqualified resource reference or a reference starting with
	 * "bundle:"
	 */
	public Resource getResource(String location) {
	  Assert.notNull(location, "location is required");
	  if (location.startsWith(BUNDLE_URL_PREFIX)) {
	    return OsgiResourceUtils.getResourceFromBundle(location.substring(BUNDLE_URL_PREFIX.length()), osgiBundle);
	  } else if (location.startsWith(ResourceLoader.CLASSPATH_URL_PREFIX)) {
	    return OsgiResourceUtils.getResourceFromBundleClasspath(location.substring(ResourceLoader.CLASSPATH_URL_PREFIX.length()),
	        osgiBundle);
	  } else if (OsgiResourceUtils.isRelativePath(location)) {
	    return OsgiResourceUtils.getResourceFromBundleClasspath(location, osgiBundle);
	  }
	  // Understand relative file URL's since by default they are taken from the bundle
	  else if (location.startsWith(ResourceUtils.FILE_URL_PREFIX)
	    && OsgiResourceUtils.isRelativePath(location.substring(ResourceUtils.FILE_URL_PREFIX.length()))) {
	    return new FileSystemResource(location.substring(ResourceUtils.FILE_URL_PREFIX.length()));
	  }
	  else {
	    return super.getResource(location);
	  }
	}

	/* (non-Javadoc)
   * @see org.springframework.context.support.AbstractApplicationContext#postProcessBeanFactory(org.springframework.beans.factory.config.ConfigurableListableBeanFactory)
   */
	protected void postProcessBeanFactory(ConfigurableListableBeanFactory beanFactory) throws BeansException {
	  super.postProcessBeanFactory(beanFactory);
	  beanFactory.addBeanPostProcessor(new BundleContextAwareProcessor(this.osgiBundleContext));
	  beanFactory.ignoreDependencyInterface(BundleContextAware.class);
	}

	protected void publishContextAsOsgiService() {
	  Dictionary serviceProperties = new Properties();
	  serviceProperties.put(APPLICATION_CONTEXT_SERVICE_NAME_HEADER, getServiceName());

	  this.serviceRegistration =
	      this.osgiBundleContext.registerService(new String[]{ApplicationContext.class.getName()},
	          this,
	          serviceProperties);
	}

	public void close() {
	  try {
	    if (this.serviceRegistration != null) {
	      this.serviceRegistration.unregister();
	    }
	  } catch (IllegalStateException alreadyUnregisteredException) {
	    // TODO: I don't like silently swallowing this
	    // since we don't expect anyone else to have unregistered the service,
	    // we should probably log this (nothing more severe since we're shutting down
	    // anyway).
	  }
	  super.close();
	}

	public boolean isAvailable() {
		return true;
	}

	protected static ApplicationContext getParentApplicationContext(BundleContext context) {
		String parentContextServiceName = (String) context.getBundle().getHeaders().get(PARENT_CONTEXT_SERVICE_NAME_HEADER);

		if (parentContextServiceName == null) {
			return null;
		} else {
			// try to find the service
			String filter = "(" + APPLICATION_CONTEXT_SERVICE_NAME_HEADER +
					"=" + parentContextServiceName + ")";
			ServiceReference ref = OsgiServiceUtils.getService(context, ApplicationContext.class, filter);
			ApplicationContext parent = (ApplicationContext) context.getService(ref);
			// TODO: register as service listener..., probably in a proxy to the app context
			// that we create here and return instead.

			return parent;
		}
	}
}
