/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
package org.springframework.osgi.context.support;

import org.springframework.beans.factory.xml.NamespaceHandlerResolver;
import org.springframework.beans.factory.xml.NamespaceHandler;
import org.osgi.framework.ServiceReference;
import org.osgi.framework.BundleContext;
import org.osgi.framework.InvalidSyntaxException;

/**
 * Resolves NamespaceHandlers by delegating to plugins which have registered as an OSGi service
 * @author Hal Hildebrand
 */
class ServiceNamespaceResolver implements NamespaceHandlerResolver
{
  private ServiceReference[] resolvers;
  private BundleContext bundleContext;
  private NamespaceHandlerResolver delegate;

  public ServiceNamespaceResolver(BundleContext bundleContext) {
    this(bundleContext, null);
  }

  public ServiceNamespaceResolver(BundleContext bundleContext, NamespaceHandlerResolver delegate) {
    this.bundleContext = bundleContext;
    this.delegate = delegate;
    try {
      resolvers = this.bundleContext.getAllServiceReferences(NamespaceHandlerResolver.class.getName(), null);
	    if (resolvers == null) throw new IllegalStateException("No NamespaceHandlerResolvers could be found.");
    } catch (InvalidSyntaxException e) {
      throw new AssertionError(e); // Will never happen
    }
  }

  public NamespaceHandler resolve(String namespaceUri) {
    for (int i = 0; i < resolvers.length; i++) {
      NamespaceHandlerResolver resolver = (NamespaceHandlerResolver) this.bundleContext
          .getService(resolvers[i]);
      try {
        NamespaceHandler source = resolver.resolve(namespaceUri);
        if (source != null) {
          return source;
        }
      } catch (IllegalArgumentException e) {
        // This is thrown when the DefaultNamespaceHandlerResolver is unable to resolve the namespace
      } finally {
        this.bundleContext.ungetService(resolvers[i]);
      }
    }
    if (delegate != null) {
      return delegate.resolve(namespaceUri);
    }
    return null;
  }
}
