/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
package org.springframework.osgi.context.support;

import java.io.IOException;

import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.osgi.framework.ServiceReference;
import org.osgi.framework.BundleContext;
import org.osgi.framework.InvalidSyntaxException;

/**
 * Resolves schemas by delegating to plugins which have registered as  an OSGi service
 * @author Hal Hildebrand
 */
class ServiceSchemaResolver implements EntityResolver
{
  private ServiceReference[] resolvers;
  private BundleContext bundleContext;
  private EntityResolver delegate;

  public ServiceSchemaResolver(BundleContext bundleContext) {
    this(bundleContext, null);
  }

  public ServiceSchemaResolver(BundleContext bundleContext, EntityResolver delegate) {
    this.bundleContext = bundleContext;
    this.delegate = delegate;
    try {
      resolvers = this.bundleContext.getAllServiceReferences(EntityResolver.class.getName(), null);
	    if (resolvers == null) throw new IllegalStateException("No EntityResolvers could be found.");
    } catch (InvalidSyntaxException e) {
      throw new AssertionError(e); // Will never happen
    }
  }

  public InputSource resolveEntity(String publicId, String systemId) throws SAXException, IOException {
      for (int i = 0; i < resolvers.length; i++) {
       EntityResolver resolver = (EntityResolver) this.bundleContext.getService(resolvers[i]);
       try {
         InputSource source = resolver.resolveEntity(publicId, systemId);
         if (source != null) {
           return source;
         }
       } finally {
         this.bundleContext.ungetService(resolvers[i]);
       }
      }
    if (delegate != null) {
      return delegate.resolveEntity(publicId, systemId);
    }
    return null;
  }
}
