/*
 * Copyright 2002-2006 the original author or authors.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Created on 23-Jan-2006 by Adrian Colyer
 */
package org.springframework.osgi.service;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;
import java.util.Set;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceFactory;
import org.osgi.framework.ServiceRegistration;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.beans.factory.BeanFactoryAware;
import org.springframework.beans.factory.BeanNameAware;
import org.springframework.beans.factory.DisposableBean;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.NoSuchBeanDefinitionException;
import org.springframework.beans.factory.support.BeanDefinitionRegistry;
import org.springframework.osgi.context.BundleContextAware;
import org.springframework.util.ClassUtils;
import org.springframework.util.StringUtils;

/**
 * A bean that transparently publishes other beans in the same
 * application context as OSGi services.
 * <p/>
 * The service properties used when publishing the service are
 * determined by the OsgiServicePropertiesResolver. The default
 * implementation uses
 * <ul>
 * <li>BundleSymbolicName=&lt;bundle symbolic name&gt;</li>
 * <li>BundleVersion=&lt;bundle version&gt;</li>
 * <li>org.springframework.osgi.beanname="&lt;bean name&gt;</li>
 * </ul>
 *
 * @author Adrian Colyer
 * @since 2.0
 */
public class OsgiServiceExporter implements BeanFactoryAware, BeanNameAware, InitializingBean, DisposableBean, BundleContextAware
{

	private Log log = LogFactory.getLog(OsgiServiceExporter.class);

	private BundleContext bundleContext;
	private OsgiServicePropertiesResolver resolver = new BeanNameServicePropertiesResolver();
	private BeanFactory beanFactory;
	private List/*<String>*/ exportBeans = Collections.EMPTY_LIST;
	private Set/*<ServiceRegistration>*/ publishedServices = new HashSet();
	private Map serviceProperties;
	private String beanName = OsgiServiceExporter.class.getName();
	private String ref;
	private String serviceInterface;
	private String activationMethod;
	private String deactivationMethod;

	/* (non-Javadoc)
	 * @see org.springframework.beans.factory.BeanFactoryAware#setBeanFactory(org.springframework.beans.factory.BeanFactory)
	 */
	public void setBeanFactory(BeanFactory beanFactory) throws BeansException {
		this.beanFactory = beanFactory;
	}

	public void setExportBeans(List /*<String>*/ beanNames) {
		if (beanNames == null) {
			this.exportBeans = Collections.EMPTY_LIST;
		} else {
			this.exportBeans = beanNames;
			for (Iterator iter = beanNames.iterator(); iter.hasNext();) {
				Object thisName = iter.next();
				if (! (thisName instanceof String)) {
					throw new IllegalArgumentException(
							"The exportBeans property requires a list of bean names as strings, " +
									"but the list contained an element of type '" + thisName.getClass().getName() +
									"'");
				}

			}
		}
	}

	public void setBundleContext(BundleContext context) {
		this.bundleContext = context;
		// REVIEW andyp -- when coming through the ContextLoaderListener this seems necessary.
		if (resolver instanceof BundleContextAware) {
			((BundleContextAware) resolver).setBundleContext(context);
		}
	}

	/**
	 * @return Returns the resolver.
	 */
	public OsgiServicePropertiesResolver getResolver() {
		return this.resolver;
	}

	/**
	 * @param resolver The resolver to set.
	 */
	public void setResolver(OsgiServicePropertiesResolver resolver) {
		this.resolver = resolver;
	}

	/* (non-Javadoc)
	 * @see org.springframework.beans.factory.BeanNameAware#setBeanName(java.lang.String)
	 */
	public void setBeanName(String name) {
		this.beanName = name;
	}

	public String getRef() {
		return ref;
	}

	public void setRef(String ref) {
		this.ref = ref;
	}

	public String getInterface() {
		return serviceInterface;
	}

	public void setInterface(String serviceInterface) {
		this.serviceInterface = serviceInterface;
	}

	public String getActivationMethod() {
		return activationMethod;
	}

	public void setActivationMethod(String activationMethod) {
		this.activationMethod = activationMethod;
	}

	public String getDeactivationMethod() {
		return deactivationMethod;
	}

	public void setDeactivationMethod(String deactivationMethod) {
		this.deactivationMethod = deactivationMethod;
	}

	/* (non-Javadoc)
	   * @see org.springframework.beans.factory.InitializingBean#afterPropertiesSet()
	   */
	public void afterPropertiesSet() throws Exception {
		if (this.beanFactory == null) {
			throw new IllegalArgumentException("Required property beanFactory has not been set");
		}
		if (this.bundleContext == null) {
			throw new IllegalArgumentException("Required property bundleContext has not been set");
		}
		if (this.resolver == null) {
			throw new IllegalArgumentException("Required property resolver was set to a null value");
		}
		if (this.exportBeans.isEmpty() && (ref == null || ref.length() == 0)) {
			if (log.isWarnEnabled()) {
				log.warn("OsgiServiceExporter '" + this.beanName + "' has no beans to export." +
						"Use the exportBeans or ref properties to specify the bean names that should be exported.");
			}
		}
		if (this.ref != null && this.ref.length() > 0
				&& (this.serviceInterface == null || this.serviceInterface.length() == 0)) {
			throw new IllegalArgumentException("Required interface has not been set");
		}
		publishBeans();
	}

	/* (non-Javadoc)
	 * @see org.springframework.beans.factory.DisposableBean#destroy()
	 */
	public void destroy() throws Exception {
		for (Iterator iter = this.publishedServices.iterator(); iter.hasNext();) {
			ServiceRegistration sReg = (ServiceRegistration) iter.next();
			try {
				sReg.unregister();
			}
			catch (IllegalStateException ise) {
				// Service was already unregistered, probably because the bundle was stopped.
				if (log.isInfoEnabled()) {
					log.info("Service [" + sReg + "] has already been unregistered");
				}
			}
		}
	}

	private void publishBeans() throws NoSuchBeanDefinitionException {
		for (Iterator iter = this.exportBeans.iterator(); iter.hasNext();) {
			String name = (String) iter.next();
			publishBeanAsService(name, this.resolver.getServiceProperties(name));
		}
		// publish the ref also if set
		if (ref != null && ref.length() > 0) {
			publishService(getInterface(), ref, this.resolver.getServiceProperties(ref));
		}
	}

	protected void publishBeanAsService(String bean, Properties serviceProperties) {
		Class[] interfaces = ClassUtils.getAllInterfaces(bean);
		if (interfaces != null && interfaces.length > 0) {
			publishService(interfaces[0].getName(), bean, serviceProperties);
		} else {
			publishService(bean.getClass().getName(), bean, serviceProperties);
		}
	}

	protected ServiceRegistration publishService(String name, String beanName, Properties serviceProperties) {
		if (log.isInfoEnabled()) {
			log.info("Publishing service [" + name + "]");
		}
		// Service registration optionally proxied to avoid eager creation
		ServiceRegistration s;
		if (beanFactory.containsBean("&" + beanName)
				|| (beanFactory instanceof BeanDefinitionRegistry)
				&& ((BeanDefinitionRegistry) beanFactory).getBeanDefinition(beanName).isLazyInit()) {
			s = bundleContext.registerService(name, new BeanServiceFactory(beanName), serviceProperties);
		} else {
			s = bundleContext.registerService(name, beanFactory.getBean(beanName), serviceProperties);
		}
		this.publishedServices.add(s);
		return s;
	}

	private class BeanServiceFactory implements ServiceFactory
	{
		private String beanName;

		public BeanServiceFactory(String beanName) {
			this.beanName = beanName;
		}

		public Object getService(Bundle bundle, ServiceRegistration serviceRegistration) {
			Object bean = beanFactory.getBean(beanName);
			if (StringUtils.hasText(activationMethod)) {
				Method m = BeanUtils.resolveSignature(activationMethod, beanFactory.getType(beanName));
				try {
					m.invoke(bean, null);
				} catch (IllegalAccessException e) {
					throw new IllegalArgumentException(e);
				} catch (InvocationTargetException e) {
					log.error("Activation method on bean with name '" + beanName + "' threw an exception", e.getTargetException());
				}
			} else if (bean instanceof ServiceActivationLifecycleListener) {
				((ServiceActivationLifecycleListener) bean).activate(serviceRegistration.getClass());
			}
			return bean;
		}

		public void ungetService(Bundle bundle, ServiceRegistration serviceRegistration, Object bean) {
			if (StringUtils.hasText(deactivationMethod)) {
				Method m = BeanUtils.resolveSignature(deactivationMethod, beanFactory.getType(beanName));
				try {
					m.invoke(bean, null);
				} catch (IllegalAccessException e) {
					throw new IllegalArgumentException(e);
				} catch (InvocationTargetException e) {
					log.error("Deactivation method on bean with name '" + beanName + "' threw an exception", e.getTargetException());
				}
			} else if (bean instanceof ServiceDeactivationLifecycleListener) {
				((ServiceDeactivationLifecycleListener) bean).deactivate(serviceRegistration.getClass());
			}
		}
	}
}
