/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.springframework.jms.support.destination;

import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.Session;

import org.springframework.jms.support.JmsAccessor;

/**
 * Base class for {@link org.springframework.jms.core.JmsTemplate} and other
 * JMS-accessing gateway helpers, adding destination-related properties to
 * {@link JmsAccessor JmsAccessor's} common properties.
 *
 * <p>Not intended to be used directly. See {@link org.springframework.jms.core.JmsTemplate}.
 *
 * @author Juergen Hoeller
 * @since 1.2.5
 * @see org.springframework.jms.support.JmsAccessor
 * @see org.springframework.jms.core.JmsTemplate
 */
public class JmsDestinationAccessor extends JmsAccessor {

	private boolean pubSubDomain = false;

	private DestinationResolver destinationResolver = new DynamicDestinationResolver();


	/**
	 * Configure the destination accessor with knowledge of the JMS domain used.
	 * Default is Point-to-Point (Queues).
	 * <p>For JMS 1.0.2 based accessors, this tells the JMS provider which class hierarchy
	 * to use in the implementation of its operations. For JMS 1.1 based accessors, this
	 * setting does usually not affect operations. However, for both JMS versions, this
	 * setting tells what type of destination to create if dynamic destinations are enabled.
	 * @param pubSubDomain "true" for the Publish/Subscribe domain ({@link javax.jms.Topic Topics}),
	 * "false" for the Point-to-Point domain ({@link javax.jms.Queue Queues})
	 * @see #setDestinationResolver
	 */
	public void setPubSubDomain(boolean pubSubDomain) {
		this.pubSubDomain = pubSubDomain;
	}

	/**
	 * Return whether the Publish/Subscribe domain ({@link javax.jms.Topic Topics}) is used.
	 * Otherwise, the Point-to-Point domain ({@link javax.jms.Queue Queues}) is used.
	 */
	public boolean isPubSubDomain() {
		return pubSubDomain;
	}

	/**
	 * Set the destination resolver that is to be used to resolve
     * {@link Destination Destinations} for this accessor.
	 * <p>The default resolver is a {@link DynamicDestinationResolver}. Specify a
	 * {@link JndiDestinationResolver} for resolving destination names as JNDI locations.
     * @param destinationResolver the destination resolver that is to be used
	 * @see org.springframework.jms.support.destination.DynamicDestinationResolver
	 * @see org.springframework.jms.support.destination.JndiDestinationResolver
	 */
	public void setDestinationResolver(DestinationResolver destinationResolver) {
		this.destinationResolver = destinationResolver;
	}

	/**
	 * Return the destination resolver for this accessor.
     * @return the destination resolver for this accessor
	 */
	public DestinationResolver getDestinationResolver() {
		return destinationResolver;
	}


	/**
	 * Resolve the given destination name into a JMS {@link Destination},
	 * via this accessor's {@link DestinationResolver}.
	 * @param session the current JMS {@link Session}
	 * @param destinationName the name of the destination
	 * @return the located {@link Destination}
	 * @throws javax.jms.JMSException if resolution failed
	 * @see #setDestinationResolver
     * @see #getDestinationResolver() 
	 */
	protected Destination resolveDestinationName(Session session, String destinationName) throws JMSException {
		return getDestinationResolver().resolveDestinationName(session, destinationName, isPubSubDomain());
	}


    /**
	 * Validate configuration.
	 */
	public void afterPropertiesSet() {
		super.afterPropertiesSet();

		if (this.destinationResolver == null) {
			throw new IllegalArgumentException("destinationResolver is required");
		}
	}

}
